/*********************************************************************************/
/* Optimized and multithreaded matrix product                                    */
/* S. Vialle - october 2014                                                      */
/*********************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <omp.h>

//on Kyle :
#include <cblas.h>
//on Sarah :
//#include <openblas/cblas.h>

#include "main.h"
#include "Calcul.h"


/*-------------------------------------------------------------------------------*/
/* Matrix products (different kernels)                                           */
/*-------------------------------------------------------------------------------*/
void LocalMatrixProduct()
{
  int i, j, k;

  // Fixe le nbr de thread d'OpenMP
  omp_set_num_threads(NbThreads);

  switch (KernelId) {

  // Kernel 0 : Code optimise' implante' par le developpeur de l'application
  // - OPTIMISEZ SERIELLEMENT CE CODE "NAIF" en suivant le sujet du TP
  // - PUIS PARALLELISEZ LE AVEC OpenMP
  case 0 :
    for (i = 0; i < SIZE; i++) {
      for (j = 0; j < SIZE; j++) {
        for (k = 0; k < SIZE; k++) {
           C[i][j] += A[i][k] * B[k][j];
        }
      }
    }
    break;

  // Kernel 1 : Code trs optimise' implante' dans une bilbiotheque "BLAS"
  // - PARALLELISEZ CET APPEL AUX "BLAS" AVEC OpenMP
  case 1 :
    cblas_dgemm(CblasRowMajor, CblasNoTrans, CblasNoTrans,
                SIZE, SIZE, SIZE,
                1.0, &A[0][0], SIZE, 
                &B[0][0], SIZE,
                0.0, &C[0][0], SIZE);
    break;

  default :
    fprintf(stderr,"Error: kernel %d not implemented!\n",KernelId);
    exit(EXIT_FAILURE);
    break;
  }
}
